@@ -72,6 +72,8 @@ from gc3libs.utils import read_contents
 from gc3libs.cmdline import SessionBasedScript
 from gc3libs.workflow import ParallelTaskCollection, TaskCollection
 from gc3libs.backends.shellcmd import ShellcmdLrms
+from gc3libs.backends.transport import LocalTransport
+
 
 class TestRunner(object):
     """
@@ -84,7 +86,8 @@ class TestRunner(object):
         self.testdir = os.path.join(self.appdir, 'test')
         self.passed = False
         kw['environment'] = {'PYTHONUNBUFFERED': 'yes'}
-        kw['output_dir'] = os.path.join(kw.get('output_dir', ''), os.path.basename(appdir))
+        kw['output_dir'] = os.path.join(
+            kw.get('output_dir', ''), os.path.basename(appdir))
         kw['stdout'] = str(self).lower()+'.stdout.log'
         kw['stderr'] = str(self).lower()+'.stderr.log'
         self.stdargs = ['-s', 'TEST_SESSION',
@@ -93,7 +96,13 @@ class TestRunner(object):
                         '-vvvv']
 
     def compatible_resources(self, resources):
-        return [r for r in resources if isinstance(r, ShellcmdLrms)]
+        shellcmd = [r for r in resources if isinstance(r, ShellcmdLrms)]
+        shellcmd_local = [r for r in shellcmd if
+                          isinstance(r.transport, LocalTransport)]
+        if shellcmd_local:
+            return shellcmd_local
+        else:
+            return shellcmd
 
 
 class GCodemlTest(TestRunner, Application):
@@ -104,19 +113,21 @@ class GCodemlTest(TestRunner, Application):
         TestRunner.__init__(self, appdir, kw)
         self.datadir = os.path.join(self.testdir, 'data/small_flat')
         Application.__init__(
-                         self,
-                         arguments=['./gcodeml.py'] + self.stdargs + ['small_flat'],
-                         inputs=[
-                             os.path.join(self.appdir, 'gcodeml.py'),
-                             self.datadir],
-                         outputs=['small_flat.out'],
-                         **kw)
+            self,
+            arguments=['./gcodeml.py'] +
+            self.stdargs + ['small_flat'],
+            inputs=[
+                os.path.join(self.appdir, 'gcodeml.py'),
+                self.datadir],
+            outputs=['small_flat.out'],
+            **kw)
 
     def terminated(self):
         self.outdir = os.path.join(self.output_dir, 'small_flat.out')
 
         if not os.path.isdir(self.outdir):
-            gc3libs.log.error("GCodeml: No output directory `%s` found" % self.outdir)
+            gc3libs.log.error("GCodeml: No output directory `%s` found",
+                              self.outdir)
             self.passed = False
             return
 
@@ -124,14 +135,16 @@ class GCodemlTest(TestRunner, Application):
             if ctlfile.endswith('.ctl'):
                 outfile = os.path.join(self.outdir, ctlfile[:-3]+'mlc')
                 if not os.path.exists(outfile):
-                    gc3libs.log.error("GCodeml: No output file %s" % outfile)
+                    gc3libs.log.error("GCodeml: No output file %s",
+                                      outfile)
                     self.passed = False
                     return
                 fd = open(outfile)
                 output = fd.readlines()
                 fd.close()
                 if not output or not output[-1].startswith("Time used:"):
-                    gc3libs.log.error("GCodeml: Error in output file `%s`" % outfile)
+                    gc3libs.log.error("GCodeml: Error in output file `%s`",
+                                      outfile)
                     self.passed = False
                     return
 
@@ -147,9 +160,8 @@ class GGamessTest(TestRunner, Application):
 
         Application.__init__(
             self,
-            arguments=['./ggamess.py' ] + self.stdargs + [
-                     '-R', '2012R1',
-                     'exam01.inp'],
+            arguments=['./ggamess.py'] + self.stdargs + ['-R', '2012R1',
+                                                         'exam01.inp'],
             inputs=[os.path.join(self.appdir, 'ggamess.py'),
                     os.path.join(self.appdir, 'test/data/exam01.inp')],
             outputs=['exam01'],
@@ -164,10 +176,13 @@ class GGamessTest(TestRunner, Application):
             self.passed = False
         stdout = read_contents(examfile)
 
-        if re.match('.*\n (EXECUTION OF GAMESS TERMINATED NORMALLY).*', stdout, re.M|re.S):
+        if re.match('.*\n (EXECUTION OF GAMESS TERMINATED NORMALLY).*',
+                    stdout, re.M | re.S):
             self.passed = True
         else:
-            gc3libs.log.error("GGamess: Output file %s does not match success regexp" % examfile)
+            gc3libs.log.error(
+                "GGamess: Output file %s does not match success regexp",
+                examfile)
             self.passed = False
 
 
@@ -179,14 +194,14 @@ class GGeotopTest(TestRunner, Application):
         TestRunner.__init__(self, appdir, kw)
         Application.__init__(
             self,
-            arguments=['./ggeotop.py'] + self.stdargs + [
-                       '-x', 'geotop_1_224_20120227_static',
-                       'GEOtop_public_test' ],
+            arguments=['./ggeotop.py'] + self.stdargs \
+                + ['-x', 'geotop_1_224_20120227_static',
+                   'GEOtop_public_test'],
             inputs=[
-                os.path.join(self.appdir, name) for name in [
-                    'ggeotop.py',
-                    'test/geotop_1_224_20120227_static',
-                    'test/data/GEOtop_public_test']],
+                os.path.join(self.appdir, name) for name in
+                ['ggeotop.py',
+                 'test/geotop_1_224_20120227_static',
+                 'test/data/GEOtop_public_test']],
             outputs=['GEOtop_public_test/out'],
             **kw)
 
@@ -202,7 +217,7 @@ class GZodsTest(TestRunner, Application):
         TestRunner.__init__(self, appdir, kw)
         Application.__init__(
             self,
-            arguments=['./gzods.py' ] + self.stdargs + ['small'],
+            arguments=['./gzods.py'] + self.stdargs + ['small'],
             inputs=[
                 os.path.join(self.appdir, i) for i in [
                     'gzods.py', 'test/data/small']],
@@ -220,20 +235,21 @@ class GRosettaTest(TestRunner, Application):
 
     def __init__(self, appdir, **kw):
         TestRunner.__init__(self, appdir, kw)
-        self.jobdirs = [ '0--1', '2--3', '4--5']
-        kw['output_dir'] = os.path.join(os.path.dirname(kw['output_dir']), 'rosetta')
+        self.jobdirs = ['0--1', '2--3', '4--5']
+        kw['output_dir'] = os.path.join(
+            os.path.dirname(kw['output_dir']), 'rosetta')
 
         Application.__init__(
             self,
-            arguments=['./grosetta.py' ] + self.stdargs + [
-                '--total-decoys', '5',
-                '--decoys-per-job', '2',
-                'data/grosetta.flags',
-                'data/alignment.filt',
-                'data/boinc_aaquery0*',
-                'data/query.*',
-                'data/*.pdb',
-                ],
+            arguments=['./grosetta.py'] + self.stdargs \
+                + ['--total-decoys', '5',
+                   '--decoys-per-job', '2',
+                   'data/grosetta.flags',
+                   'data/alignment.filt',
+                   'data/boinc_aaquery0*',
+                   'data/query.*',
+                   'data/*.pdb',
+                   ],
             inputs=[os.path.join(self.appdir, 'grosetta.py'),
                     os.path.join(self.appdir, 'test/data')],
             outputs=self.jobdirs + ['data'],
@@ -242,7 +258,7 @@ class GRosettaTest(TestRunner, Application):
     def terminated(self):
         for jobdir in self.jobdirs:
             if not os.path.isdir(jobdir):
-                gc3libs.log.error("GRosetta: missing job directory %s" % jobdir)
+                gc3libs.log.error("GRosetta: missing job directory %s", jobdir)
                 self.passed = False
                 return
             outfile = os.path.join(jobdir, 'minirosetta.static.stdout.txt')
@@ -255,13 +271,17 @@ class GRosettaTest(TestRunner, Application):
             output = fd.readlines()
             fd.close()
             try:
-                if not output or output[-1] != 'minirosetta.static: All done, exitcode: 0':
-                    gc3libs.log.error("GRosetta: expecting different output in `%s`. Last line is: `%s`" % (outfile, output[-1]))
+                if not output or output[-1] != \
+                        'minirosetta.static: All done, exitcode: 0':
+                    gc3libs.log.error(
+                        "GRosetta: expecting different output in `%s`. "
+                        "Last line is: `%s`", outfile, output[-1])
                     self.passed = False
                     return
             except Exception, ex:
                 gc3libs.log.error(
-                    "GRosetta: Error while checking exit status of `grosetta`: %s" % str(ex))
+                    "GRosetta: Error while checking exit status of `grosetta`:"
+                    " %s", str(ex))
                 self.passed = False
                 raise ex
         self.passed = True
@@ -273,17 +293,18 @@ class GDockingTest(TestRunner, Application):
 
     def __init__(self, appdir, **kw):
         TestRunner.__init__(self, appdir, kw)
-        kw['output_dir'] = os.path.join(os.path.dirname(kw['output_dir']), 'docking')
+        kw['output_dir'] = os.path.join(
+            os.path.dirname(kw['output_dir']), 'docking')
 
         self.jobdirs = ["1bjpA.%s" % d for d in ('1--2', '3--4', '5--5')]
         Application.__init__(
             self,
-            arguments=['./gdocking.py' ] + self.stdargs + [
-                     '--decoys-per-file', '5',
-                     '--decoys-per-job', '2',
-                     '-f', 'data/gdocking.flags',
-                     'data/1bjpA.pdb',
-                     ],
+            arguments=['./gdocking.py'] + self.stdargs \
+                + ['--decoys-per-file', '5',
+                   '--decoys-per-job', '2',
+                   '-f', 'data/gdocking.flags',
+                   'data/1bjpA.pdb',
+                   ],
             inputs=[
                 os.path.join(self.appdir, 'gdocking.py'),
                 os.path.join(self.appdir, 'test/data')],
@@ -293,17 +314,38 @@ class GDockingTest(TestRunner, Application):
     def terminated(self):
         for jobdir in self.jobdirs:
             if not os.path.isdir(jobdir):
-                gc3libs.log.error("GDocking: missing job directory %s" % jobdir)
+                gc3libs.log.error(
+                    "GDocking: missing job directory %s" % jobdir)
                 self.passed = False
                 return
             outfile = os.path.join(jobdir, 'docking_protocol.stdout.txt')
 
             if not os.path.isfile(outfile):
-                gc3libs.log.error("GDocking: missing output file %s" % outfile)
+                gc3libs.log.error(
+                    "GDocking: missing output file %s" % outfile)
                 self.passed = False
                 return
         self.passed = True
 
+class GGcGpsTest(TestRunner, Application):
+    def __str__(self):
+        return "GGcGps"
+
+    def __init__(self, appdir, **kw):
+        TestRunner.__init__(self, appdir, kw)
+        Application.__init__(
+            self,
+            arguments = ['./gc_gps.py']  + self.stdargs + ['small.txt', 'src', '-i', 'in', '-o', 'out'],
+            inputs = [
+                os.path.join(self.appdir, name) for name in
+                ('gc_gps.py', 'test-gc_gps/small.txt', 'test-gc_gps/src', 'test-gc_gps/in')],
+            outputs=['out'],
+            **kw)
+
+    def terminated(self):
+        self.passed = self.execution._exitcode == 0
+
+
 class GCryptoTest(TestRunner, Application):
     def __str__(self):
         return "GCrypto"
@@ -316,15 +358,15 @@ class GCryptoTest(TestRunner, Application):
                   os.path.join(self.testdir, 'input.tgz'),
                   os.path.join(self.testdir, 'gnfs-cmd_20120406'),
                   ]
-        if os.path.isdir(self.testdir) and min(os.path.isfile(f) for f in ifiles):
+        if os.path.isdir(self.testdir) \
+                and min(os.path.isfile(f) for f in ifiles):
             # input files found. Run gcrypto test.
             args = [
                 './gcrypto.py',
                 '-i', 'input.tgz',
                 '-g', 'gnfs-cmd_20120406',
-                '-c', '2'] + self.stdargs + [
-                '800000000', '800001000', '500',
-                ]
+                '-c', '2'] + self.stdargs \
+                + ['800000000', '800001000', '500', ]
 
         Application.__init__(
             self,
@@ -339,21 +381,20 @@ class GCryptoTest(TestRunner, Application):
 
 
 class RunTestsInParallel(ParallelTaskCollection):
-    applicationdirs = {
-        # 'bf.uzh.ch': None,
-        'codeml': (GCodemlTest,),
-        # 'compchem.unipg.it': None,
-        'gamess': (GGamessTest,),
-        # 'gc3.uzh.ch': None,
-        'geotop': (GGeotopTest, ),
-        # 'ieu.uzh.ch': (GMhCoevTest, ),
-        # 'imsb.ethz.ch': None,
-        # 'ior.uzh.ch': None,
-        'lacal.epfl.ch': (GCryptoTest, ),
-        'rosetta': (GRosettaTest, GDockingTest),
-        # 'turbomole': None,
-        'zods': (GZodsTest, ),
-        }
+    applicationdirs = {'codeml': (GCodemlTest,),
+                       'gamess': (GGamessTest,),
+                       'geotop': (GGeotopTest, GGcGpsTest,),
+                       'lacal.epfl.ch': (GCryptoTest, ),
+                       'rosetta': (GRosettaTest, GDockingTest),
+                       'zods': (GZodsTest, ),
+                       # 'bf.uzh.ch': None,
+                       # 'compchem.unipg.it': None,
+                       # 'gc3.uzh.ch': None,
+                       # 'ieu.uzh.ch': (GMhCoevTest, ),
+                       # 'imsb.ethz.ch': None,
+                       # 'ior.uzh.ch': None,
+                       # 'turbomole': None,
+                       }
 
     def __init__(self, tests=None, **extra):
         """
@@ -363,13 +404,18 @@ class RunTestsInParallel(ParallelTaskCollection):
         if not tests:
             tests = self.applicationdirs
         else:
-            tests = dict((k,v) for k,v in self.applicationdirs.iteritems() if k in tests)
+            tests = dict((k, v) for k, v in self.applicationdirs.iteritems()
+                         if k in tests)
         tasks = []
         extra['output_dir'] = "RunTestAppsInParallel"
         for testdir, classes in tests.iteritems():
             appdir = os.path.abspath(testdir)
 
-            tasks+= [cls(appdir, **extra) for cls in classes if issubclass(cls, Task) and issubclass(cls, TestRunner)]
+            tasks += [
+                cls(appdir, **extra) for cls in classes
+                if issubclass(cls, Task) and issubclass(cls, TestRunner)]
+        if not tasks:
+            raise RuntimeError("No tasks found")
         ParallelTaskCollection.__init__(self, tasks, **extra)
 
 
@@ -380,27 +426,29 @@ class TestAppsScript(SessionBasedScript):
     version = '1.0'
 
     def setup_args(self):
-        self.add_param('args',
-                       nargs='*',
-                       metavar='TESTS',
-                       default=RunTestsInParallel.applicationdirs.keys(),
-                       help="If no `TEST` is given all configured tests will be run. If one or more"
-                       " directories are given from command line, only "
-                       "tests configured for those directories will be "
-                       "run.")
+        self.add_param(
+            'args',
+            nargs='*',
+            metavar='TESTS',
+            default=RunTestsInParallel.applicationdirs.keys(),
+            help="If no `TEST` is given all configured tests will be run. "
+            "If one or more directories are given from command line, only "
+            "tests configured for those directories will be run.")
 
     def setup_options(self):
-        self.add_param('-R', '--test-resource', dest='test_resource', metavar='RESOURCE',
+        self.add_param('-R', '--test-resource',
+                       dest='test_resource', metavar='RESOURCE',
                        required=True,
-                       help="Run test script so that they are submitted to `RESOURCE` resource, "
-                       "by addiong option `-r RESOURCE` to the gc3apps script.")
+                       help="Run test script so that they are submitted to "
+                       "`RESOURCE` resource, "
+                       "by adding option `-r RESOURCE` to the gc3apps "
+                       "script.")
 
     def new_tasks(self, extra):
         extra_args = extra.copy()
         extra_args['resource'] = self.params.test_resource
         return [RunTestsInParallel(self.params.args, **extra_args)]
 
-
     def get_tasks(self):
         for task in self.session.tasks.values():
             if isinstance(task, TaskCollection):
@@ -410,17 +458,31 @@ class TestAppsScript(SessionBasedScript):
                 yield t
 
     def before_main_loop(self):
-        print "Tests passed from command line: %s" % str.join(", ", self.params.args)
-        lrms = self._core.get_backend(self.params.test_resource)
+        print "Tests passed from command line: %s" % str.join(", ",
+                                                              self.params.args)
         # The resource may requires a password to be properly
-        # initialized (e.g. x509 proxy certificates).
-        lrms.get_resource_status()
+        # initialized (e.g. x509 proxy certificates). This is usually
+        # done during submission, but when the application is
+        # submitted to, e.g., smscg, we don't have a prompt anymore,
+        # so let's try now and check that the resource is available.
+        lrms = self._core.get_backend(self.params.test_resource)
+        if not lrms.enabled:
+            lrms.enabled = True
+        self._core.update_resources()
+        if not lrms.updated:            
+            raise RuntimeError("Resource '%s' not updated. Exiting" % lrms.name)
 
-    def after_main_loop(self):
-        print "Status of applications:"
+    def every_main_loop(self):
+        print
+        print "Current status of applications:"
 
         for task in self.get_tasks():
-            print "Task %s: %s (passed: %s)" % (task, task.execution.state, task.passed)
+            print "Task %s: %s (passed: %s)" % (task,
+                                                task.execution.state,
+                                                task.passed)
+        print
+
+    after_main_loop = every_main_loop
 
 if __name__ == "__main__":
     from run_test_apps import TestAppsScript
